package Model;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.StringTokenizer;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;

import Control.Controller;
import Control.IdleThread;
import Control.InputCommands;
import Model.Direction.InitType;
import View.View;

/**
 * Application osztly.
 * 
 * Ezt hozza ltre elszr a modellek kzl a Controller, felels a szintek
 * betltsrt, a jtklls mentsrt, egy korbban mentett lls
 * betltsrt, a program futsnak befejezsrt. Mozgsra utastja a
 * csempket vagy a figurt.
 * 
 * @author Szab Csaba
 * @author Gerlei Balzs
 * @author Halsz Antal Istvn
 * @author Berki Ramna
 * @version 1.0
 */
public class Application implements ActionListener {

	/** Az ppen betlttt szint. */
	private Level actualLevel;
	
	private boolean firstSimulate;

	/** A szintek elrsi tjt trolja. */
	private ArrayList<String> levelsPath;

	/** rtke az aktulis szint indexe a levelsPath tmbben. */
	private int levelsPathIndex;

	/** Az elz simulateWorld futsi ideje. */
	private int previousTime;

	/**
	 * Az elmentett, vagy pp betlttt szint indexe. Null, ha nincs mentett
	 * szint.
	 */
	private int actualLevelIndex;
	
	public String getActualLevelPath() {
		return levelsPath.get(actualLevelIndex);
	}

	/** Application konstruktora. Ltrehoz egy j Applicationt. */
	public Application() {
		levelsPath = new ArrayList<String>();
		actualLevel = null;
		levelsPathIndex = 0;
		actualLevelIndex = 0;
		previousTime = 0;
		firstSimulate = true;
		
		levelsPath.add("szint1.txt");
		levelsPath.add("szint2.txt");
		levelsPath.add("szint2.txt");
	}

	/** Kilp a programbl s automatikusan menti a jtk llst. */
	public void exit() {
		System.exit(0);
	}

	/**
	 * Betlti az tadott elrsi tnak megfelel szintet.
	 * 
	 * @param i
	 *            a betltend szint indexe, ha ezzel toltjuk be a palyat akkor
	 *            a levelInputPath erteke null
	 * @param levelInputPath
	 *            prototipusnal kell csak, a betoltendo szint eleresi utja, ha
	 *            innen toltjuk be a palyat akkor az i erteke -1
	 */
	public void loadLevel(int i, String levelInputPath, PrintWriter pw) {

		if (levelInputPath != null && i == -1) {
			BufferedReader br = null;

			try {
				br = new BufferedReader(new FileReader(levelInputPath));
			} catch (FileNotFoundException e1) {
				e1.printStackTrace();
			}

			// Begin-end kztt vagyunk-e
			boolean isBeginLevel = false;
			boolean isBeginFullTile = false;
			boolean isBeginField = false;

			// Melyik szintet, csempt, stb-t szerkesztjk
			int levelCount = 0;
			int fullTileCount = 0; // Figyeljl hogy a Level tiles listjban a
									// 0. az EmptyTile!
			int fieldCount = 0;

			try {

				// a fajl soronkenti beolvasasa
				while (true) {
					String line = br.readLine();
					// Exit felttel, ha nincs line
					if (line == null)
						break;

					// a beolvasott sor szavakra bontasa
					StringTokenizer token = new StringTokenizer(line);
					int counter = 0;

					// bemeneti nyelv parancsainak nyilvantartasahoz
					LevelInputCommands commandState = null;

					String param1BeginLevel = null;
					String param2BeginLevel = null;
					String param1AddFieldPoint = null;
					String param1AddKey = null;
					String param1AddExit = null;
					String param2AddExit = null;
					String param1AddFigureA = null;
					String param2AddFigureA = null;
					String param3AddFigureA = null;
					String param1AddFigureB = null;
					String param2AddFigureB = null;
					String param3AddFigureB = null;

					while (token.hasMoreTokens()) {
						line = token.nextToken();

						// parancs beolvasas, ez alapjan beallitom a commandSate
						// enum valtozot
						if (counter == 0) {

							if (line.equals("beginLevel")) {
								// van parametere, nem itt hatjuk vegre a
								// parancsot
								commandState = LevelInputCommands.beginLevel;

								// Szint szerkesztsi md megkezdse
								isBeginLevel = true;

							} else if (line.equals("endLevel")) {
								// nincs parametere, mar itt vegrehajtodik a
								// parancs
								commandState = LevelInputCommands.endLevel;

								// Szint szerkesztsi md befejezse
								isBeginLevel = false;

								if (pw != null) {
									pw.println("Szint szerkesztese befejezve.");
								}
							} else if (line.equals("beginFullTile")) {
								// nincs parametere, mar itt vegrehajtodik a
								// parancs
								commandState = LevelInputCommands.beginFullTile;

								// Telicsempe hozzadsa a kvetkez helyre
								actualLevel.addFullTile();

								// Telicsempe szerkesztsi md megkezdse
								isBeginFullTile = true;
								fullTileCount++;
								fieldCount = 0;

								if (pw != null) {
									pw.println("Teli csempe hozzaadva.");
								}
							} else if (line.equals("endFullTile")) {
								// nincs parametere, mar itt vegrehajtodik a
								// parancs
								commandState = LevelInputCommands.endFullTile;

								// Ha ksz a Teli Csempe, megcsinljuk a
								// kereteit is.
								FullTile tempTile = (FullTile) actualLevel
										.getTiles().get(fullTileCount);
								// tiles tmbben els mindig az EmptyTile

								for (int i2 = 0; i2 < 4; i2++) {
									// Jobb oldali keretbe olyan pontok
									// kerlnek, amelyek x koordintja 500
									if (i2 == 0) {
										Frame right = new Frame();
										// Vgignzzk a terepeket
										for (int j = 0; j < tempTile
												.getFields().size(); j++) {

											// Vgig nzzk a terep pontjait
											// (Polygon pontjainak szmt
											// npoints
											// adja vissza)
											for (int j2 = 0; j2 < tempTile
													.getFields().get(j)
													.getFieldPoints().npoints; j2++) {
												// Ha a terepnek van a jobb
												// oldali keretbe tartoz
												// pontja, akkor hozzadjuk azt
												// kerethez
												// Polygon xpoints-knt
												// visszaadja az x pontjait
												// tartalmaz tmbt
												if ((tempTile.getFields()
														.get(j)
														.getFieldPoints().xpoints[j2]) == 500) {
													right.setFramePoint(new Point(
															0,
															tempTile.getFields()
																	.get(j)
																	.getFieldPoints().ypoints[j2]));
												}
											}
										}
										tempTile.getFrames().add(right);
									}

									// Fenti keretbe olyan pontok kerlnek,
									// amelyek y koordintja 400
									if (i2 == 1) {
										Frame up = new Frame();
										// Vgignzzk a terepeket
										for (int j = 0; j < tempTile
												.getFields().size(); j++) {

											// Vgig nzzk a terep pontjait
											for (int j2 = 0; j2 < tempTile
													.getFields().get(j)
													.getFieldPoints().npoints; j2++) {
												// Ha a terepnek van a fenti
												// keretbe tartoz pontja, akkor
												// hozzadjuk azt kerethez
												if ((tempTile.getFields()
														.get(j)
														.getFieldPoints().ypoints[j2]) == 400) {
													up.setFramePoint(new Point(
															tempTile.getFields()
																	.get(j)
																	.getFieldPoints().xpoints[j2],
															0));
												}
											}
										}
										tempTile.getFrames().add(up);
									}

									// Bal oldali keretbe olyan pontok kerlnek,
									// amelyek x koordintja 0
									if (i2 == 2) {
										Frame left = new Frame();
										// Vgignzzk a terepeket
										for (int j = 0; j < tempTile
												.getFields().size(); j++) {

											// Vgig nzzk a terep pontjait
											for (int j2 = 0; j2 < tempTile
													.getFields().get(j)
													.getFieldPoints().npoints; j2++) {
												// Ha a terepnek van a fenti
												// keretbe tartoz pontja, akkor
												// hozzadjuk azt kerethez
												if ((tempTile.getFields()
														.get(j)
														.getFieldPoints().xpoints[j2]) == 0) {
													left.setFramePoint(new Point(
															0,
															tempTile.getFields()
																	.get(j)
																	.getFieldPoints().ypoints[j2]));
												}
											}
										}
										tempTile.getFrames().add(left);
									}

									// Lenti keretbe olyan pontok kerlnek,
									// amelyek y koordintja 0
									if (i2 == 3) {
										Frame down = new Frame();
										// Vgignzzk a terepeket
										for (int j = 0; j < tempTile
												.getFields().size(); j++) {

											// Vgig nzzk a terep pontjait
											for (int j2 = 0; j2 < tempTile
													.getFields().get(j)
													.getFieldPoints().npoints; j2++) {
												// Ha a terepnek van a fenti
												// keretbe tartoz pontja, akkor
												// hozzadjuk azt kerethez
												if ((tempTile.getFields()
														.get(j)
														.getFieldPoints().ypoints[j2]) == 0) {
													down.setFramePoint(new Point(
															tempTile.getFields()
																	.get(j)
																	.getFieldPoints().xpoints[j2],
															0));
												}
											}
										}
										tempTile.getFrames().add(down);
									}

								}

								// Telicsempe szerkesztsi md befejezse
								isBeginFullTile = false;

								if (pw != null) {
									pw.println("Teli csempe szerkesztese befejezve.");
								}
							} else if (line.equals("beginField")) {
								// nincs parametere, mar itt vegrehajtodik a
								// parancs
								commandState = LevelInputCommands.beginField;

								// Terep szerkesztsi md megkezdse
								isBeginField = true;
								fieldCount++;

								// a fields ArrayList elkeszul a FullTile
								// konstruktoraban, tehat itt mar nem kell
								// letrehozni
								// csak az arrayList-hez hozzaadni egy fieldet
								FullTile temp = (FullTile) actualLevel
										.getTiles().get(fullTileCount);
								temp.getFields().add(new Field());

								if (pw != null) {
									pw.println("Uj terepalakzat szerkesztese.");
								}
							} else if (line.equals("endField")) {
								// nincs parametere, mar itt vegrehajtodik a
								// parancs
								commandState = LevelInputCommands.endField;

								// Terep szerkesztsi md befejezse
								isBeginField = false;

								if (pw != null) {
									pw.println("Terepalakzat szerkesztese befejezve.");
								}
							} else if (line.equals("addFieldPoint")) {
								// van parametere, nem itt hatjuk vegre a
								// parancsot
								if (isBeginField) {
									commandState = LevelInputCommands.addFieldPoint;
								} else {
									System.out.println("Csak beginField es"
											+ "endField kozott hivhato!");
								}

							} else if ((line.equals("addKey"))
									&& isBeginFullTile) {
								// van parametere, nem itt hatjuk vegre a
								// parancsot
								if (isBeginFullTile) {
									commandState = LevelInputCommands.addKey;
								} else {
									System.out.println("Csak beginFullTile es"
											+ "endFullTile kozott hivhato!");
								}

							} else if (line.equals("addExit")) {
								// van parametere, nem itt hatjuk vegre a
								// parancsot
								if (isBeginFullTile) {
									commandState = LevelInputCommands.addExit;
								} else {
									System.out.println("Csak beginFullTile es"
											+ "endFullTile kozott hivhato!");
								}

							} else if (line.equals("addFigureA")) {
								// van parametere, nem itt hatjuk vegre a
								// parancsot
								if (isBeginLevel) {
									commandState = LevelInputCommands.addFigureA;
								} else {
									System.out.println("Csak beginLevel es"
											+ "endLevel kozott hivhato!");
								}

							} else if ((line.equals("addFigureB"))
									&& isBeginLevel) {
								// van parametere, nem itt hatjuk vegre a
								// parancsot
								if (isBeginLevel) {
									commandState = LevelInputCommands.addFigureB;
								} else {
									System.out.println("Csak beginLevel es"
											+ "endLevel kozott hivhato!");
								}
							}
						}
						// parameter beolvasasa
						else {

							switch (commandState) {
							case beginLevel:
								// 3 darab parameter
								// 1. parameter: A palya neve
								// 2. parameter: Hanyszor hanyas matrix alapjan
								// epul fel a Level, X koordinata
								// 3. parameter: Hanyszor hanyas matrix alapjan
								// epul fel a Level, Y koordinata
								if (counter == 1) {
									param1BeginLevel = line;
								} else if (counter == 2) {
									param2BeginLevel = line;
								} else {
									if (pw != null) {
										pw.println(param1BeginLevel
												+ " szint letrehozasa sikeres.");
									}
									actualLevel = new Level(this,
											Integer.parseInt(param2BeginLevel),
											Integer.parseInt(line));
									// rescsempe hozzadsa
									actualLevel.addEmptyTile();

								}
								break;

							case addFieldPoint:
								// 2 darab parameter
								// 1. parameter: Uj kontrollpont X koordinataja.
								// 2. parameter: Uj kontrollpont Y koordinataja.
								if (counter == 1) {
									param1AddFieldPoint = line;
								} else {
									// TODO: terephez pont hozzaadas
									// implementalasa es meghivasa

									FullTile temp = (FullTile) actualLevel
											.getTiles().get(fullTileCount);
									temp.addFieldPoint(
											fieldCount - 1,
											new Point(
													Integer.parseInt(param1AddFieldPoint),
													Integer.parseInt(line)));

									if (pw != null) {
										pw.println("Kontrollpont "
												+ param1AddFieldPoint + " "
												+ line + " hozzaadva.");
									}
								}
								break;

							case addKey:
								// 2 darab parameter
								// 1. parameter: A kulcs X koordinataja.
								// 2. parameter: A kulcs Y koordinataja.
								if (counter == 1) {
									param1AddKey = line;
								} else {
									FullTile tempTile2 = (FullTile) actualLevel
											.getTiles().get(fullTileCount);
									// A csempket tartalmaz kollekciban els
									// mindig az EmptyTile

									// Kulcs hozzadsa a paramterknt kapott
									// koordintkra
									tempTile2.getGameObjects().add(
											new Key(new Point(Integer
													.parseInt(param1AddKey),
													Integer.parseInt(line)), tempTile2));

									if (pw != null) {
										pw.println("Kulcs " + param1AddKey
												+ " " + line
												+ " helyen hozzaadva.");
									}
								}
								break;

							case addExit:
								// 3 darab parameter
								// 1. parameter: A kijarat X koordinataja.
								// 2. parameter: A kijarat Y koordinataja.
								// 3. parameter: Hany kulcs kell a kijarathoz.
								if (counter == 1) {
									param1AddExit = line;
								} else if (counter == 2) {
									param2AddExit = line;
								} else {
									FullTile tempTile3 = (FullTile) actualLevel
											.getTiles().get(fullTileCount);
									// A csempket tartalmaz kollekciban els
									// mindig az EmptyTile

									// Kijrat hozzadsa a paramterknt kapott
									// koordintkra
									tempTile3
											.getGameObjects()
											.add(new Exit(
													new Point(
															Integer.parseInt(param1AddExit),
															Integer.parseInt(param2AddExit)),
													Integer.parseInt(line),tempTile3));

									if (pw != null) {
										pw.println(line + " kulcsos kijarat "
												+ param1AddExit + " "
												+ param2AddExit
												+ " helyen hozzaadva.");
									}
								}
								break;

							case addFigureA:
								// 4 darab parameter
								// 1. parameter: Melyik csempebe tesszuk a
								// babut, X koordinata.
								// 2. parameter: Melyik csempebe tesszuk a
								// babut, Y koordinata.
								// 3. parameter: A csempebe hova keruljon a
								// babu, X koordinata.
								// 4. parameter: A csempebe hova keruljon a
								// babu, Y koordinata.
								if (counter == 1) {
									param1AddFigureA = line;
								} else if (counter == 2) {
									param2AddFigureA = line;
								} else if (counter == 3) {
									param3AddFigureA = line;
								} else {
									// A Figura csempjnek indexe
									int tileIndex = 0;

									// Elszr t kell vltanunk a beadott
									// csempe koordintkat csempe indexre
									// ( (beadott X koordinta) - 1) x
									// (csempemtrix szlessgt) adjuk az
									// indexhez
									for (int j = 0; j < (Integer
											.parseInt(param1AddFigureA)) - 1; j++) {
										tileIndex += (actualLevel.getY());
									}
									// majd hozzadjuk a beadott Y koordintt
									for (int j = 0; j < Integer
											.parseInt(param2AddFigureA); j++) {
										tileIndex++;
									}
									tileIndex--; // Mert 0-tl indexelnk

									// Belltjuk a szintben a bbu kezd
									// csempjt s kezd koordintjt
									actualLevel
											.setFigureAStartTileIndex(tileIndex);
									actualLevel.setFigureAStartCoord(new Point(
											Integer.parseInt(param3AddFigureA),
											Integer.parseInt(line)));
									// Bbu A-t tartalmaz csempe
									FullTile tempTile4 = (FullTile) actualLevel
											.getTiles().get(tileIndex);
									// Bbu beraksa a csempbe a paramterek
									// szerinti koordintba
									tempTile4
											.setFigureA(new Figure(
													tempTile4,
													new Point(
															Integer.parseInt(param3AddFigureA),
															Integer.parseInt(line))));

									if (pw != null) {
										pw.println("FigureA a(z) "
												+ param1AddFigureA + " "
												+ param2AddFigureA
												+ " pozicioju teli csempeben "
												+ param3AddFigureA + " " + line
												+ " pozicioba hozzaadva");
									}
								}
								break;

							case addFigureB:
								// 4 darab parameter
								// 1. parameter: Melyik csempebe tesszuk a
								// babut, X koordinata.
								// 2. parameter: Melyik csempebe tesszuk a
								// babut, Y koordinata.
								// 3. parameter: A csempebe hova keruljon a
								// babu, X koordinata.
								// 4. parameter: A csempebe hova keruljon a
								// babu, Y koordinata.
								if (counter == 1) {
									param1AddFigureB = line;
								} else if (counter == 2) {
									param2AddFigureB = line;
								} else if (counter == 3) {
									param3AddFigureB = line;
								} else {
									// A Figura csempjnek indexe
									int tileIndex = 0;

									// Elszr t kell vltanunk a beadott
									// csempe koordintkat csempe indexre
									// ( (beadott X koordinta) - 1) x
									// (csempemtrix szlessgt) adjuk az
									// indexhez
									for (int j = 0; j < (Integer
											.parseInt(param1AddFigureB)) - 1; j++) {
										tileIndex += (actualLevel.getY());
									}
									// majd hozzadjuk a beadott Y koordintt
									for (int j = 0; j < Integer
											.parseInt(param2AddFigureB); j++) {
										tileIndex++;
									}
									tileIndex--; // Mert 0-tl indexelnk

									// Belltjuk a szintben a bbu kezd
									// csempjt s kezd koordintjt
									actualLevel
											.setFigureBStartTileIndex(tileIndex);
									actualLevel.setFigureBStartCoord(new Point(
											Integer.parseInt(param3AddFigureB),
											Integer.parseInt(line)));
									// Bbu B-t tartalmaz csempe
									FullTile tempTile4 = (FullTile) actualLevel
											.getTiles().get(tileIndex);
									// Bbu beraksa a csempbe a paramterek
									// szerinti koordintba
									tempTile4
											.setFigureB(new Figure(
													tempTile4,
													new Point(
															Integer.parseInt(param3AddFigureB),
															Integer.parseInt(line))));

									if (pw != null) {
										pw.println("FigureB a(z) "
												+ param1AddFigureB + " "
												+ param2AddFigureB
												+ " pozicioju teli csempeben "
												+ param3AddFigureB + " " + line
												+ " pozicioba hozzaadva");
									}
								}
								break;
							}
						}
						counter++;

					}
				}
				br.close();
			} catch (IOException e) {
				e.printStackTrace();
			}

		}

		else if (levelInputPath == null && i != -1) {

		}

		else {
			// TODO: exception, ilyen nem fordulhat elo, rosszul lett meghivva a
			// fuggveny
		}
	}

	/**
	 * Ha van betlttt szint, akkor tovbbtja a szintnek, hogy mi mozogjon
	 * (what paramter) s milyen irnyba (dir paramter).
	 * 
	 * @param what
	 *            the what
	 * @param dir
	 *            the dir
	 */
	public void move(Type what, Direction dir, PrintWriter pw) {

		// Szerintem gy az elegns - gerbazs
		try {
			actualLevel.move(what, dir, pw);
		} catch (NullPointerException e) {
			System.out.println("Nincs betoltott szint!");
		}
	}

	/** Kinullzza a pillanatnyi plyt. */
	public void setNextLevel(PrintWriter pw) {
		actualLevelIndex++;
		
		// ha nincs tobb palya
		if(actualLevelIndex >= levelsPath.size()) {			
			i.setCanRun(false);
			JFrame win = new JFrame();
			JButton ok = new JButton("OK!");
			
			win.getContentPane().setLayout(new BoxLayout(win.getContentPane(),BoxLayout.PAGE_AXIS));
			win.getContentPane().add(new JLabel("Gratulalok! Nyertel!"));
			win.getContentPane().add(ok);
			ok.addActionListener(this);
			
			win.setBounds(100, 100, 100, 100);
			win.pack();
	        win.setVisible(true);
	        
			
		} else {
			loadLevel(-1, levelsPath.get(actualLevelIndex), pw);
			
			v = new View(getActualLevel(), c);
			c.setBounds(0, 0, 800, 600);
			v.setFocusable(true);
			v.requestFocus();
			v.addKeyListener(c);
			i.setMyView(v);
			c.getContentPane().add(v);
		}
	}

	/**
	 * Az id mlst szimullja, a betlttt szint gravitcit kezel
	 * fggvnyt hvja, ha van betlttt szint.
	 * 
	 * @param time
	 *            the time
	 */
	public void simulateWorld(int time, PrintWriter pw) {
		if(firstSimulate == true){
			firstSimulate = false;
			return;
		}
		
		int dt = time - previousTime;

		try {
			actualLevel.simulateWorld((int)time/8, pw);
		} catch (NullPointerException e) {
			System.out.println("Nincs betoltott szint!");
		}

		previousTime = time;
	}

	/**
	 * Az eppen betoltott szintet kerdezhetjuk le.
	 */
	public Level getActualLevel() {
		return actualLevel;

	}
	
	private Controller c;
	private View v;
	private IdleThread i;
	
	public void setupView(View view, Controller controller, IdleThread iThread) {
		view = new View(getActualLevel(), controller);
		controller.setBounds(0, 0, 800, 600);
		view.setFocusable(true);
		view.requestFocus();
		view.addKeyListener(controller);
		iThread.setMyView(view);
		controller.getContentPane().add(view);
		
		v = view;
		c = controller;
		i = iThread;
	}

	public void actionPerformed(ActionEvent arg0) {
		System.exit(1);
		
	}
}
