package Model;

import java.lang.Math;

/**
 * Direction osztly.
 * 
 * Segdosztly, mely az irnyvektort reprezentlja.
 * 
 * @author Szab Csaba
 * @author Gerlei Balzs
 * @author Halsz Antal Istvn
 * @author Berki Ramna
 * @version 1.0
 */
public class Direction {

	/** Az irnyvektor szge. */
	private double angle;

	/** Az irnyvektor hossza. */
	private double length;

	/** Az irnyvektor x koordintj sszetevje. */
	private double x;

	/** Az irnyvektor y koordintj sszetevje. */
	private double y;

	/**
	 * Begyazott enum a konstruktorhoz.
	 */
	public enum InitType {

		/** The Angle. */
		Angle,
		/** The Coordinates. */
		Coordinates
	}

	/**
	 * Direction paramteres konstruktora. Ktfle mdon hozhat ltre j
	 * Directiont: a bemen paramterek az irnyvektor szge s hossza, vagy a
	 * bemen paramterek az irnyvektor x s y irny sszetevi. A vlasztst
	 * a begyazott InitType enum megadsval egyrtelmstjk: Angle, ha szg
	 * s hossz alapjn, Coordinates, ha pedig koordintk alapjn
	 * inicializlunk.
	 * 
	 * @param a_x
	 *            az irnyvektor szge fokban vagy x irny komponense
	 * @param l_y
	 *            az irnyvektor hossza vagy y irny komponense
	 * @param init
	 *            a vlasztott inicializlsi mdot jelz enum
	 */
	public Direction(double a_x, double l_y, InitType init) {
		if (init == InitType.Angle) {
			angle = a_x;
			length = l_y;
			if (angle == 0.0) {
				x = length;
				y = 0.0;
			} else if (angle == 90.0) {
				x = 0.0;
				y = length;
			} else if (angle == 180.0) {
				x = -length;
				y = 0.0;
			} else if (angle == 270.0) {
				x = 0.0;
				y = -length;
			} else {
				x = Math.cos(Math.toRadians(a_x)) * length;
				y = Math.sin(Math.toRadians(a_x)) * length;
			}
		} else if (init == InitType.Coordinates) {
			x = a_x;
			y = l_y;
			length = Math.sqrt(x * x + y * y);
			// a szg init kicsit trkks a Math fggvnyei miatt
			if (Math.abs(x) <= 0.015) {
				x = 0.0;
				if (Math.abs(y) >= 0.015 && y > 0) {
					angle = 90;
					return;
				} else if (Math.abs(y) >= 0.015 && y < 0) {
					angle = 270;
					return;
				} else {
					angle = 0;
					length = 0;
					return;
				}
			} else if (Math.abs(y) <= 0.015) {
				y = 0.0;
				if (Math.abs(x) >= 0.015 && x > 0) {
					angle = 0;
					return;
				} else if (Math.abs(x) >= 0.015 && x < 0) {
					angle = 180;
					return;
				} else {
					angle = 0;
					length = 0;
					return;
				}
			} else {
				angle = Math.toDegrees(Math.atan(y / x));
				if (x < 0)
					angle += 180;
				if (angle >= 360)
					angle -= 360;
				if (angle <= 360)
					angle += 360;
			}
		} else {
			System.out.println("Direction letrehozasi hiba!");
		}
	}

	/**
	 * Direction default konstruktora. Ltrehoz egy j directiont
	 * nullvektorknt.
	 */
	public Direction() {
		angle = 0.0;
		length = 0.0;
		x = 0.0;
		y = 0.0;
	}
	
	public Direction(Direction other) {
		angle = other.angle;
		length = other.length;
		x = other.x;
		y = other.y;
	}

	/**
	 * Irnyvektor vektor sszead, egy j vektorba sszeadja magt s egy msik
	 * vektort.
	 * 
	 * @param other
	 *            A hozzadand irnyvektor.
	 * @return sszegzett irny.
	 */
	public Direction add(Direction other) {
		return new Direction(this.x + other.x, this.y + other.y,
				InitType.Coordinates);
	}

	/**
	 * Visszaadja az irnyvektor szgt.
	 * 
	 * @return irnyvektor szge
	 */
	public double getAngle() {
		return angle;
	}

	/**
	 * Visszaadja az irnyvektor hosszt.
	 * 
	 * @return az irnyvektor hossza
	 */
	public double getLength() {
		return length;
	}

	/**
	 * Visszaadja az irnyvektor x koordintj sszetevjt.
	 * 
	 * @return az x koordintj sszetev
	 */
	public double getX() {
		return x;
	}

	/**
	 * Visszaadja az irnyvektor y koordintj sszetevjt.
	 * 
	 * @return az y koordintj sszetev
	 */
	public double getY() {
		return y;
	}

	/**
	 * Belltja az irnyvektor szgt.
	 * 
	 * @param a
	 *            az j szg
	 */
	public void setAngle(double a) {
		angle = a;
		if (angle == 0.0) {
			x = length;
			y = 0.0;
		} else if (angle == 90.0) {
			x = 0.0;
			y = length;
		} else if (angle == 180.0) {
			x = -length;
			y = 0.0;
		} else if (angle == 270.0) {
			x = 0.0;
			y = -length;
		} else {
			x = Math.cos(Math.toRadians(angle)) * length;
			y = Math.sin(Math.toRadians(angle)) * length;
		}
	}

	/**
	 * Belltja az irnyvektor hosszt.
	 * 
	 * @param l
	 *            az j hosszsg
	 */
	public void setLength(double l) {
		if (Math.abs(l) >= 0.015) {
			length = l;
			x = Math.cos(Math.toRadians(angle)) * l;
			y = Math.sin(Math.toRadians(angle)) * l;
		} else {
			angle = 0;
			length = 0;
			x = 0;
			y = 0;
		}
	}

	/**
	 * Belltja az irnyvektor x koordintj sszetevjt.
	 * 
	 * @param newX
	 *            az j x koordintj sszetev
	 */
	public void setX(double newX) {
		x = newX;
		if (Math.abs(x) <= 0.015) {
			x = 0.0;
		}
		length = Math.sqrt(x * x + y * y);
		angle = Math.toDegrees(Math.acos(x / length));
		// trkkzs a Math fgv-k miatt
		if (y < 0)
			angle = 360 - angle;
	}

	/**
	 * Belltja az irnyvektor y koordintj sszetevjt.
	 * 
	 * @param newY
	 *            az j y koordintj sszetev
	 */
	public void setY(double newY) {
		y = newY;
		if(Math.abs(y) <= 0.015){
			y = 0.0;
		}
		length = Math.sqrt(x * x + y * y);
		angle = Math.toDegrees(Math.asin(y / length));
		// trkkzs a Math fgv-k miatt
		if (x < 0)
			angle = 180 - angle;
	}
}
