package Model;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
import java.io.*;
import java.util.*;

/**
 * Field osztly.
 * 
 * A terepet reprezentlja. Meg tudja mondani azt, hogy egy adott pontban van-e
 * terep.
 * 
 * @author Szab Csaba
 * @author Gerlei Balzs
 * @author Halsz Antal Istvn
 * @author Berki Ramna
 * @version 1.0
 */
public class Field {
	/**
	 * A terepet reprezentl pontok. A terepen egyenesek vannak. Egy egyenes
	 * kt vgpontjt egy Point collection trolja index alapjn sorban. Tbb
	 * Point tmb (egyenes) reprezentlja a terepet.
	 */
	private Polygon points;

	/**
	 * Field konstruktora. Ltrehoz egy j terepet.
	 */
	public Field() {
		// Plya inicializlsa
		points = new Polygon();
	}

	public void addPoint(Point p) {
		points.addPoint(p.x, p.y);
	}

	/**
	 * Referencit ad a terepalakzatot ler poligonra.
	 * 
	 * @return a terepalakzatot ler poligon
	 */
	public Polygon getFieldPoints() {
		return points;
	}

	/**
	 * Megmondja, hogy a paramterknt kapott pontban terep van-e.
	 * 
	 * @param p
	 *            az ellenrizend koordinta
	 * @return igaz, ha van terep az adott koordintn
	 */
	public boolean isThereField(Point p) {
		return points.contains(p);
	}

	/**
	 * A modell beli koordintt a kirajzolsnak megfelel koordinta-rendszerbe
	 * vltja t.
	 * 
	 * @return kirajzolshoz hasznlhat koordinta
	 */
	public Polygon transformToMatrixCoord() {
		int[] temp = new int[points.npoints];
		for (int i = 0; i < points.npoints; ++i) {
			temp[i] = 400 - points.ypoints[i];
		}
		Polygon transformed = new Polygon(points.xpoints, temp,
				points.npoints);

		return transformed;
	}
}
