package Model;

import java.awt.Point;
import java.io.PrintWriter;
import java.util.*;

import Model.FullTile;

/**
 * Level osztly.
 * 
 * Felels a csempk elrendezsrt, s a bbu kvnt irny mozgsnak
 * belltsrt. Meg tudja mondani az adott szint kezdcsempjt s a bbu azon
 * belli kezdpontjt.
 * 
 * @author Szab Csaba
 * @author Gerlei Balzs
 * @author Halsz Antal Istvn
 * @author Berki Ramna
 * @version 1.0
 */
public class Level {
	/**
	 * Referencia az alkalmazsra, ami ltrehozta. A kvetkez szint
	 * betltsekor hasznljuk.
	 */
	private Application app;

	/** Az 1. bbu kezdcsempn belli kezdkoordinti. */
	private Point figureAStartCoord;
	/** A 2. bbu kezdcsempn belli kezdkoordinti. */
	private Point figureBStartCoord;

	/** Az 1. bbu kezdcsempjnek indexe a tiles tmbben. */
	private int figureAStartTileIndex;
	/** A 2. bbu kezdcsempjnek indexe a tiles tmbben. */
	private int figureBStartTileIndex;

	/**
	 * A szinten lv csempk. A 0. index elem az egyetlen res csempe
	 * (EmptyTile).
	 */
	private ArrayList<Tile> tiles;

	/** Hanyszor hanyas matrix alapjan epulnek fel a csempek (tiles). */
	int x, y;

	/**
	 * Level konstruktora. Ltrehoz egy j szintet.
	 * 
	 * @param a
	 *            referencia a szl applicationra
	 * @param x0
	 *            the x0
	 * @param y0
	 *            the y0
	 */
	public Level(Application a, int x0, int y0) {
		app = a;
		tiles = new ArrayList<Tile>();
		x = x0;
		y = y0;
	}

	/**
	 * res csempe hozzadsa.
	 */
	public void addEmptyTile() {
		tiles.add(new EmptyTile(this, new Point(1, 1)));
	}

	/**
	 * Teli csempe hozzadsa.
	 */
	public void addFullTile() {
		// hanyadik sorba es oszlopba kerul a csempe
		int fX = 1, fY = 1;

		// hanyadik helyre kerul ez a csempe amit most be akarunk rakni
		int count = tiles.size() + 1;

		// megnezzuk hogy melyik sorba fog kerulni
		while (count > y) {
			count -= y;
			fX++;
		}
		fY = count;
		tiles.add(new FullTile(this, new Point(fX, fY)));
	}

	/**
	 * Prototipusban, kirajzolja a csempek jelenlegi helyzetet.
	 * 
	 * @param pw
	 *            Fjlba r printwriter.
	 */
	public void drawTiles(PrintWriter pw) {
		if (pw != null) {
			if (y == 2) pw.println("--------------------------");
			if (y == 3) pw.println("-------------------------------------------");
		}
		for (int i = 0; i < tiles.size(); i++) {
			int pX = 1, pY = 1;
			int count = i + 1;

			while (count > y) {
				count -= y;
				pX++;
			}
			pY = count;

			for (int j = 0; j < tiles.size(); j++) {
				if (tiles.get(j).getTileIndex().equals(new Point(pX, pY))) {
					if (j == 0) {
						if (pw != null)
							pw.print("EmptyTile");

					} else {
						if (pw != null)
							pw.print("FullTile" + j);
					}
					if (pY != y)
						if (pw != null)

							pw.print("        ");
						else {
							if (pw != null)
								pw.println();
						}

					if ((i + 1) % y == 0)
						pw.println();

					break;
				}
			}

		}
		if (pw != null){
			if (y == 2) pw.println("--------------------------");
			if (y == 3) pw.println("-------------------------------------------");
		}
		
	}

	/**
	 * Jelzi az Application fel, hogy a szint teljestve lett.
	 */
	public void finished(PrintWriter pw) {
		app.setNextLevel(pw);
	}

	/**
	 * Visszaad egy referencit az res csempre.
	 * 
	 * @return the empty tile
	 */
	public EmptyTile getEmptyTile() {
		return (EmptyTile) tiles.get(0);
	}

	/**
	 * Referencit ad vissza az 1. bbu kezd koordintira.
	 * 
	 * @return az 1. bbu kezd koordintja
	 */
	public Point getFigureAStartCoord() {
		return figureAStartCoord;
	}

	/**
	 * Referencit ad vissza a 2. bbu kezd koordintira.
	 * 
	 * @return a 2. bbu kezd koordintja
	 */
	public Point getFigureBStartCoord() {
		return figureBStartCoord;
	}

	/**
	 * Visszaadja az 1. bbu kezdcsempjnek indext.
	 * 
	 * @return az 1. bbu kezd csempjnek indexe
	 */
	public int getFigureAStartTileIndex() {
		return figureAStartTileIndex;
	}

	/**
	 * Visszaadja a 2. bbu kezdcsempjnek indext.
	 * 
	 * @return a bbu kezd csempjnek indexe
	 */
	public int getFigureBStartTileIndex() {
		return figureBStartTileIndex;
	}

	/**
	 * Referencit ad az argumentumok alapjn keresett csempre. Egy csempe
	 * adott oldali szomszdjnak kidertsre hasznljuk.
	 * 
	 * @param dir
	 *            melyik irnyban lv szomszd kell
	 * @param pos
	 *            hv pozcija
	 * @return a krt szomszd
	 */
	public FullTile getNeighbour(Direction dir, Point pos) {

		Point neighbourPos = new Point(pos.x, pos.y);

		if (dir.getAngle() == 0) {
			neighbourPos.y++;
		} else if (dir.getAngle() == 90) {
			neighbourPos.x--;
		} else if (dir.getAngle() == 180) {
			neighbourPos.y--;
		} else if (dir.getAngle() == 270) {
			neighbourPos.x++;
		}

		for (int i = 1; i < tiles.size(); i++) {
			if (tiles.get(i).getTileIndex().x == neighbourPos.x
					&& tiles.get(i).getTileIndex().y == neighbourPos.y) {
				return (FullTile) tiles.get(i);
			}
		}
		return null;
	}

	/**
	 * Visszadja a csempk gyjtemnyt.
	 * 
	 * @return Referencia a csempk gyjtemnyrl.
	 */
	public ArrayList<Tile> getTiles() {
		return tiles;
	}

	public int getX() {
		return x;
	}

	public int getY() {
		return y;
	}

	/**
	 * Megkapja, hogy mi mozogjon, s ennek hvja meg a mozgat metdust,
	 * aminek tadja a kapott irnyt (dir).
	 * 
	 * @param what
	 *            , bbut vagy csempt
	 * @param dir
	 *            , mozgats irnya
	 * @param pw
	 *            a kir
	 */
	public void move(Type what, Direction dir, PrintWriter pw) {
		if (what == Type.FigureA) {
			for (int i = 1; i < tiles.size(); ++i) {
				FullTile f = (FullTile) tiles.get(i);
				if (f.isThereFigureA()) {
					f.changeFigureADirection(dir);
				}
			}
		} else if (what == Type.FigureB) {
			for (int i = 1; i < tiles.size(); ++i) {
				FullTile f = (FullTile) tiles.get(i);
				if (f.isThereFigureB()) {
					f.changeFigureBDirection(dir);
				}
			}

		} else if (what == Type.Tile) {
			EmptyTile e = this.getEmptyTile();
			e.move(dir, pw);
		}
	}

	/**
	 * Belltja a Bbu A start koordintjt
	 * 
	 * @param p
	 *            A belltand koordinta.
	 */
	public void setFigureAStartCoord(Point p) {
		figureAStartCoord = p;
	}

	/**
	 * Belltja a Bbu B start koordintjt
	 * 
	 * @param p
	 *            A belltand koordinta.
	 */
	public void setFigureBStartCoord(Point p) {
		figureBStartCoord = p;
	}

	/**
	 * Belltja az 1. bbu kezd csempjt.
	 * 
	 * @param index
	 *            a kezd csempe indexe a tiles tmbben
	 */
	public void setFigureAStartTileIndex(int index) {
		figureAStartTileIndex = index;
	}

	/**
	 * Belltja a 2. bbu kezd csempjt.
	 * 
	 * @param index
	 *            a kezd csempe indexe a tiles tmbben
	 */
	public void setFigureBStartTileIndex(int index) {
		figureBStartTileIndex = index;
	}

	/**
	 * Tovbbadja az idt annak a FullTile-nak, amiben a bbu van.
	 * 
	 * @param dt
	 *            the dt
	 * @param pw
	 *            the pw
	 */
	public void simulateWorld(int dt, PrintWriter pw) {
		boolean figureAMoved = false;
		boolean figureBMoved = false;
		
		for (int i = 1; i < tiles.size(); ++i) {
			FullTile temp = (FullTile) tiles.get(i);
			if (temp.isThereFigureA() && figureAMoved == false) {
				temp.simulateFigureAWorld(dt, "FigureA", pw);
				figureAMoved = true;
			}
			if (temp.isThereFigureB() && figureBMoved == false) {
				temp.simulateFigureBWorld(dt, "FigureB", pw);
				figureBMoved = true;
			}
		}
	}

}
